package com.zy.omp.common;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.zy.omp.common.enums.ResponseCode;
import lombok.Data;

import java.io.Serializable;

/**
 * 接口数据返回定义类
 */
@Data
@JsonSerialize
public class ServerResponse implements Serializable {
    //接口返回状态
    private Integer status;
    //返回提示信息
    private String msg;
    //返回的数据
    private Object data;

    /**
     * 根据成功与否及是否需要返回data提供的各种静态构造方法
     */
    public static ServerResponse createBySuccess() {
        return new ServerResponse(ResponseCode.SUCCESS.code, null, ResponseCode.SUCCESS.msg);
    }

    public static ServerResponse createBySuccess(Object data) {
        return new ServerResponse(ResponseCode.SUCCESS.code, data, ResponseCode.SUCCESS.msg);
    }

    public static ServerResponse createByError() {
        return new ServerResponse(ResponseCode.ERROR.code, null, ResponseCode.ERROR.msg);
    }

    public static ServerResponse createByWarning(String msg) {
        return new ServerResponse(ResponseCode.WARNING.code, null, msg);
    }

    public static ServerResponse createByAuthor() {
        return new ServerResponse(ResponseCode.AUTHOR.code, null, ResponseCode.AUTHOR.msg);
    }

    private ServerResponse(int status, Object data, String msg) {
        this.status = status;
        this.msg = msg;
        this.data = data;
    }
}
