package com.zy.omp.service;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zy.omp.pojo.io.pc.UserIO;
import com.zy.omp.mapper.UserMapper;
import com.zy.omp.model.User;
import com.zy.omp.utils.HttpUtil;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;

/**
 * 用户Service
 *
 * @author chenyi
 * Create on 2020/4/10
 */
@Service
public class UserService extends ServiceImpl<UserMapper, User> {

    /**
     * 微信小程序登录相关配置
     */
    private static final String wxAppId = "wxe90699b45e28a0b6";// 小程序唯一标识
    private static final String wxSecret = "66690a1ffcfe7f1d85a6dcfa38791f25";//小程序的 app secret
    private static final String grant_type = "authorization_code"; // 授权（必填）

    /**
     * 后台
     * 分页查询用户列表
     *
     * @param io 查询条件
     */
    public IPage<User> listPage(UserIO io) {
        Page<User> page = new Page<>(io.getCurrent(), io.getSize());
        return baseMapper.listPage(page, io);
    }

    /**
     * 通过 openId 查询用户
     */
    public User getByOpenId(String openId) {
        return baseMapper.selectOne(new QueryWrapper<User>().eq("openId", openId));
    }

    /**
     * 通过微信传来的code 换取 appid
     */
    public String getWxAppId(String code) {
        try {
            // 请求参数
            MultiValueMap<String, String> paramsMap = new LinkedMultiValueMap<>();
            paramsMap.add("appid", wxAppId);
            paramsMap.add("secret", wxSecret);
            paramsMap.add("js_code", code);
            paramsMap.add("grant_type", grant_type);
            String message = HttpUtil.post("https://api.weixin.qq.com/sns/jscode2session", null, paramsMap);
            return JSONObject.parseObject(message).getString("openid");
        } catch (Exception e) {
            return null;
        }
    }
}
